@@ -1,7 +1,7 @@
 /*
- * forced apm sleep daemon
+ * sleep daemon
  *
- * Copyright 2000-2006 Joey Hess <joeyh@kitenet.net> under the terms of the
+ * Copyright 2000-2008 Joey Hess <joeyh@kitenet.net> under the terms of the
  * GNU GPL.
  */
 
@@ -19,6 +19,9 @@
 #include <fcntl.h>
 #include <apm.h>
 #include "acpi.h"
+#ifdef HAL
+#include "simplehal.h"
+#endif
 #include <signal.h>
 #include "sleepd.h"
 
@@ -35,6 +38,9 @@ int have_irqs=0;
 int sleep_time = DEFAULT_SLEEP_TIME;
 int no_sleep=0;
 signed int min_batt=-1;
+#ifdef HAL
+int use_simplehal = 0;
+#endif
 int use_acpi=0;
 int require_unused_and_battery=0;	/* --and or -A option */
 
@@ -125,13 +131,11 @@ void parse_command_line (int argc, char **argv) {
 		autoprobe=1;
 }
 
-/* Keep checking the interrupts. As long as there is activity, do nothing. */
 void main_loop (void) {
 	long irq_count[MAX_IRQS]; /* holds previous counters of the irq's */
 	int activity, i, sleep_now=0, total_unused=0, do_this_one=0, probed=0;
 	int sleep_battery=0;
 	int prev_ac_line_status=0;
-	long v;
 	time_t nowtime, oldtime=0;
 	FILE *f;
 	char line[64];
@@ -140,12 +144,47 @@ void main_loop (void) {
 	
 	while (1) {
 		activity=0;
+
+		if (use_acpi) {
+			acpi_read(1, &ai);
+		}
+#ifdef HAL
+		else if (use_simplehal) {
+			simplehal_read(1, &ai);
+		}
+#endif
+		else {
+			apm_read(&ai);
+		}
+
+		if (min_batt != -1 && ai.ac_line_status != 1 && 
+		    ai.battery_percentage < min_batt &&
+		    ai.battery_status != BATTERY_STATUS_ABSENT) {
+			sleep_battery = 1;
+		}
+		if (sleep_battery && ! require_unused_and_battery) {
+			syslog(LOG_NOTICE, "battery level %d%% is below %d%%; forcing hibernation", ai.battery_percentage, min_batt);
+			if (system(hibernate_command) != 0)
+				syslog(LOG_ERR, "%s failed", hibernate_command);
+			/* This counts as activity; to prevent double sleeps. */
+			activity=1;
+			oldtime=0;
+			sleep_battery=0;
+		}
+	
+		/* Rest is only needed if sleeping on inactivity. */
+		if (! max_unused && ! ac_max_unused) {
+			sleep(sleep_time);
+			continue;
+		}
+
 		f=fopen(INTERRUPTS, "r");
 		if (! f) {
 			perror(INTERRUPTS);
 			exit(1);
 		}
 		while(fgets(line,sizeof(line),f)) {
+			long v;
 			if (autoprobe) {
 				/* Lowercase line. */
 				for(i=0;line[i];i++)
@@ -177,13 +216,6 @@ void main_loop (void) {
 				syslog(LOG_WARNING, "no keyboard or mouse irqs autoprobed");
 			}
 		}
-		
-		if (use_acpi) {
-			acpi_read(1, &ai);
-		}
-		else {
-			apm_read(&ai);
-		}
 
 		if (ai.ac_line_status != prev_ac_line_status) {
 			/* AC plug/unplug counts as activity. */
@@ -191,20 +223,6 @@ void main_loop (void) {
 		}
 		prev_ac_line_status=ai.ac_line_status;
 		
-		if (min_batt != -1 && ai.ac_line_status != 1 && 
-		    ai.battery_percentage < min_batt) {
-			sleep_battery = 1;
-		}
-		if (sleep_battery && ! require_unused_and_battery) {
-			syslog(LOG_NOTICE, "battery level %d%% is below %d%%; forcing hibernation", ai.battery_percentage, min_batt);
-			if (system(hibernate_command) != 0)
-				syslog(LOG_ERR, "%s failed", hibernate_command);
-			/* This counts as activity; to prevent double sleeps. */
-			activity=1;
-			oldtime=0;
-			sleep_battery=0;
-		}
-		
 		if (activity) {
 			total_unused = 0;
 		}
@@ -316,21 +334,29 @@ int main (int argc, char **argv) {
 	}
 	
 	if (apm_exists() != 0) {
+#ifdef HAL
+		/* this test must come before the acpi_supported test,
+		 * since that test will succeed even if acpi provides no
+		 * useful information */
+		if (simplehal_supported()) {
+			use_simplehal=1;
+			if (! sleep_command)
+				sleep_command=acpi_sleep_command;
+		}
+		else
+#endif
 		if (acpi_supported()) {
 			use_acpi=1;
+			if (! sleep_command)
+				sleep_command=acpi_sleep_command;
 		}
 		else {
-			fprintf(stderr, "sleepd: apm/acpi support not present in kernel\n");
+			fprintf(stderr, "sleepd: no APM, ACPI, or HAL support detected\n");
 			exit(1);
 		}
 	}
 	if (! sleep_command) {
-		if (use_acpi) {
-			sleep_command=acpi_sleep_command;
-		}
-		else {
-			sleep_command=apm_sleep_command;
-		}
+		sleep_command=apm_sleep_command;
 	}
 	if (! hibernate_command) {
 		hibernate_command=sleep_command;
